@@ -1,12 +1,35 @@
+from typing import Dict, MutableMapping, Optional
+
 from dependency_injector import containers, providers
 
+from physrisk.data.hazard_data_provider import SourcePath
 from physrisk.data.inventory import EmbeddedInventory
 from physrisk.data.inventory_reader import InventoryReader
 from physrisk.data.pregenerated_hazard_model import ZarrHazardModel
 from physrisk.data.zarr_reader import ZarrReader
+from physrisk.kernel.hazard_model import HazardModelFactory
 from physrisk.requests import Requester, _create_inventory, create_source_paths
 
 
+class ZarrHazardModelFactory(HazardModelFactory):
+    def __init__(
+        self,
+        source_paths: Dict[type, SourcePath],
+        store: Optional[MutableMapping] = None,
+        reader: Optional[ZarrReader] = None,
+    ):
+        self.source_paths = source_paths
+        self.store = store
+        self.reader = reader
+
+    def hazard_model(self, interpolation: str = "floor"):
+        # this is done to allow interpolation to be set dynamically, e.g. different requests can have different
+        # parameters.
+        return ZarrHazardModel(
+            source_paths=self.source_paths, store=self.store, reader=self.reader, interpolation=interpolation
+        )
+
+
 class Container(containers.DeclarativeContainer):
     config = providers.Configuration(default={"zarr_sources": ["embedded", "hazard"]})
 
@@ -22,13 +45,11 @@ class Container(containers.DeclarativeContainer):
 
     zarr_reader = providers.Singleton(ZarrReader, store=zarr_store)
 
-    hazard_model = providers.Singleton(
-        ZarrHazardModel, reader=zarr_reader, source_paths=source_paths, interpolation="floor"
-    )
+    hazard_model_factory = providers.Factory(ZarrHazardModelFactory, reader=zarr_reader, source_paths=source_paths)
 
     requester = providers.Singleton(
         Requester,
-        hazard_model=hazard_model,
+        hazard_model_factory=hazard_model_factory,
         inventory=inventory,
         inventory_reader=inventory_reader,
         reader=zarr_reader,
