@@ -1,3 +1,4 @@
+import inspect
 import sys
 from enum import Enum
 from typing import cast
@@ -19,17 +20,26 @@ class Hazard:
         return cast(HazardKind, hazard_type.kind)
 
 
+class ChronicHeat(Hazard):
+    kind = HazardKind.chronic
+    pass
+
+
 class Inundation(Hazard):
     kind = HazardKind.acute
     pass
 
 
+class AirTemperature(ChronicHeat):
+    pass
+
+
 class CoastalInundation(Inundation):
     kind = HazardKind.acute
     pass
 
 
-class ChronicHeat(Hazard):
+class ChronicWind(Hazard):
     kind = HazardKind.chronic
     pass
 
@@ -54,6 +64,11 @@ class Hail(Hazard):
     pass
 
 
+class PluvialInundation(Inundation):
+    kind = HazardKind.acute
+    pass
+
+
 class Precipitation(Hazard):
     kind = HazardKind.chronic
     pass
@@ -64,10 +79,41 @@ class RiverineInundation(Inundation):
     pass
 
 
-class Wind(Hazard):
+class WaterRisk(Hazard):
     kind = HazardKind.chronic
     pass
 
 
+class WaterDemand(WaterRisk):
+    pass
+
+
+class WaterSupply(WaterRisk):
+    pass
+
+
+class WaterStress(WaterRisk):
+    pass
+
+
+class WaterTemperature(ChronicHeat):
+    pass
+
+
+class WaterDepletion(WaterRisk):
+    pass
+
+
+class Wind(Hazard):
+    kind = HazardKind.acute
+    pass
+
+
+def all_hazards():
+    return [
+        obj for _, obj in inspect.getmembers(sys.modules[__name__]) if inspect.isclass(obj) and issubclass(obj, Hazard)
+    ]
+
+
 def hazard_class(name: str):
     return getattr(sys.modules[__name__], name)
