@@ -3,7 +3,7 @@ from typing import List, Union
 
 import numpy as np
 
-from .curve import to_exceedance_curve
+from physrisk.kernel.curve import to_exceedance_curve
 
 
 class ImpactType(Enum):
@@ -38,8 +38,17 @@ class ImpactDistrib:
         return zip(self.__impact_bins[0:-1], self.__impact_bins[1:])
 
     def mean_impact(self):
+        if len(self.__impact_bins) == len(self.__prob):
+            return np.sum(self.__impact_bins * self.__prob) + self.__impact_bins[-1] * (1.0 - np.sum(self.__prob))
         return np.sum((self.__impact_bins[:-1] + self.__impact_bins[1:]) * self.__prob / 2)
 
+    def stddev_impact(self):
+        if len(self.__impact_bins) == len(self.__prob):
+            return 0
+        mean = self.mean_impact()
+        bin_mids = (self.__impact_bins[:-1] + self.__impact_bins[1:]) / 2
+        return np.sqrt(np.sum(self.__prob * (bin_mids - mean) * (bin_mids - mean)))
+
     def to_exceedance_curve(self):
         return to_exceedance_curve(self.__impact_bins, self.__prob)
 
@@ -54,3 +63,8 @@ class ImpactDistrib:
     @property
     def prob(self) -> np.ndarray:
         return self.__prob
+
+
+class EmptyImpactDistrib(ImpactDistrib):
+    def __init__(self):
+        pass
